
% DCG grammar which outputs syntactic structure and FOL representations
% based on Pereira&Shieber (1987), p. 102-103,  program 4.2
% added extras:
%    output: syntactic structures
%    logical representations in prefix notation
%    renamed existential quantifier from 'exists' to 'some'
%    intersective adjectives
%    various lexical entries


% parse/3: parse(+String,-SyntacticStructure,-LogicalRepresentation)
% parse predicate for sentences

parse(Input,Syntax,LF) :-
	s(Syntax,LF,Input,[]).


% category(?LF);  usage: category(?LF,+list,+elist)

% Grammar Rules

s(s(Np,Vp),S)   --> np(Np,VP^S), vp(Vp,VP).


np(Str,NP)      --> det(Det,N2^NP), n(N,N1), optrel(Rel,N1^N2),
	               {(N1==N2,Str=np(Det,N),!)
		       ;(Str=np(Det,N,Rel))}.
np(Str,NP)      --> det(Det,N2^NP), n1(N,N1), optrel(Rel,N1^N2),
	               {(N1==N2,Str=np(Det,N),!)
		       ;(Str=np(Det,N,Rel))}.
np(np(Pn),(E^S)^S) --> pn(Pn,E).

n1(n1(Adj,N),N1)   --> adj(Adj,Nominal^N1), n(N,Nominal).


vp(vp(Ttv,Np1,Np2),X^S) --> ttv(Ttv,X^Y^IV1),
	                    np(Np1,IV1^IV2),
			    np(Np2,(Y^IV2)^S).
vp(vp(Tv,Np),X^S) --> tv(Tv,X^IV), np(Np,IV^S).
vp(vp(Verb),IV)   --> iv(Verb,IV).


optrel(rel(relp(that),Vp),(X^S1)^(X^and(S1,S2))) --> [that], vp(Vp,X^S2).
optrel(_,N^N)     --> [].


% Lexicon

adj(adj(A), LF) --> [A], {adj(A,LF)}.

adj(red, (X^N)^(X^and(red(X),N))).
adj(smart, (X^N)^(X^and(smart(X),N))).


det(det(D), LF) --> [D], {det(D,LF)}.

det(every, (X^S1)^(X^S2)^all(X,imp(S1,S2))).
det(a,     (X^S1)^(X^S2)^some(X,and(S1,S2))).


n(n(N),LF)   --> [N], {n(N,LF)}.

n(book, X^book(X)).
n(dog, X^dog(X)).
n(movie, X^movie(X)).
n(professor, X^professor(X)).
n(program, X^program(X)).
n(student, X^student(X)).


pn(pn(PN),LF)   --> [PN], {pn(PN,LF)}. 

pn(fido, fido).
pn(kim, kim).
pn(north_by_northwest, north_by_northwest).
pn(sandy, sandy).
pn(shrdlu, shrdlu).
pn(terry, terry).
pn(twin_peaks, twin_peaks).
pn(ulysses, ulysses).
pn(zettels_traum, zettels_traum).


ttv(v(TV),LF)   --> [TV], {ttv(TV,LF)}.

ttv(gave, X^Y^Z^(give_p(X,Y,Z))).


tv(v(TV),LF)   --> [TV], {tv(TV,LF)}.

tv(likes, X^Y^(like(X,Y))).
tv(read, X^Y^(read_p(X,Y))).
tv(wrote, X^Y^(write_p(X,Y))).


iv(v(IV),LF)   --> [IV], {iv(IV,LF)}.

iv(barks, X^bark(X)).
iv(halts, X^halt(X)).